module.exports = bootstrap

var url = require('url')
var async = require('async')
var nano = require('nano-option')

var assureConnection = require('./lib/assure-connection')
var fixAdminParty = require('./lib/fix-admin-party')
var runBootstrap = require('./lib/bootstrap')

/**
 * Bootstapping CouchDB
 * - fix admin party
 * - run couchdb-bootstrap
 **/
function bootstrap (options, callback) {
  options.config = options.config || {}

  var nanoConfig = {
    url: options.couchUrl,
    auth: options.config.admin
  }
  var state = {
    name: options.name,
    config: options.config,
    bootstrapPath: options.bootstrapPath,
    couch: getCouchConnection(nanoConfig),
    adminCouch: getAdminCouchConnection(nanoConfig),
    log: options.log || console.log.bind(console)
  }

  async.waterfall([
    assureConnection.bind(null, state),
    fixAdminParty.bind(null, state),
    runBootstrap.bind(null, state)
  ], callback)
}

function getCouchConnection (config) {
  return nano({
    url: config.url,
    parseUrl: false
  })
}

function getAdminCouchConnection (config) {
  var o = url.parse(config.url)

  if (config.auth && config.auth.username && config.auth.password) {
    o.auth = [
      encodeURIComponent(config.auth.username),
      encodeURIComponent(config.auth.password)
    ].join(':')
  }

  return nano({
    url: url.format(o),
    parseUrl: false
  })
}
