module.exports = bootstrap

var couchdbBootstrap = require('couchdb-bootstrap')

function bootstrap (state, next) {
  couchdbBootstrap(state.adminCouch, state.bootstrapPath, {
    mapDbName: function (baseName) {
      var dbMappings = {}
      var config = state.config
      var name = state.name
      var names = [name, baseName]

      for (var k in names) {
        if (config[names[k]] && config[names[k]].db) {
          var identifier
          var moduleMapping = config[names[k]].db
          for (identifier in moduleMapping) {
            if (moduleMapping.hasOwnProperty(identifier)) {
              dbMappings[identifier] = moduleMapping[identifier]
            }
          }
        }
      }

      var plugin
      for (plugin in config.plugin) {
        if (config.plugin.hasOwnProperty(plugin) && config.plugin[plugin] && config.plugin[plugin].db) {
          if (config.plugin[plugin].db[baseName]) {
            dbMappings[baseName] = config.plugin[plugin].db[baseName]
          }
        }
      }

      return dbMappings[baseName] || baseName
    }
  }, function (error, response) {
    /* istanbul ignore if */
    if (error) {
      return next(error)
    }

    /* istanbul ignore if */
    if ('configure' in response && response.configure && response.configure.length) {
      state.log([state.name, 'bootstrap', 'info'], 'config updated')
    }

    /* istanbul ignore if */
    if ('secure' in response) {
      Object.keys(response.secure).forEach(function (dbname) {
        /* istanbul ignore if */
        state.log([state.name, 'bootstrap', 'info'], '_security set on database ' + dbname + '.')
      })
    }

    /* istanbul ignore if */
    if ('push' in response) {
      Object.keys(response.push).forEach(function (dbname) {
        if (dbname === '_users') {
          var usernames = response.push._users.map(function (resp) {
            return resp.id.split(':')[1]
          })

          return state.log([state.name, 'bootstrap', 'info'], 'users created or updated: ' + usernames.join(', '))
        }

        var counts = response.push[dbname].reduce(function (memo, resp) {
          if (resp.id.match(/^_design/)) {
            memo.ddocs++
          } else {
            memo.docs++
          }
          return memo
        }, {
          ddocs: 0,
          docs: 0
        })

        if (counts.ddocs) {
          state.log([state.name, 'bootstrap', 'info'], '_design docs added or updated on database ' + dbname + ': ' + counts.ddocs + '.')
        }
        if (counts.docs) {
          state.log([state.name, 'bootstrap', 'info'], 'docs added or updated on database ' + dbname + ': ' + counts.docs + '.')
        }
      })
    }

    next(null)
  })
}
