var path = require('path')
var test = require('tap').test
var startServer = require('kazana-server')

var bootstrap = require('../..')

var serverOptions = {
  name: 'test-mod',
  plugins: [
    {
      name: 'test-plugin',
      config: {
        'test-plugin': {
          db: {
            'test-identifier': 'test-db-name'
          }
        }
      }
    }
  ],
  config: {
    kazana: {
      backend: {
        name: 'memdown',
        port: 5999
      }
    }
  }
}

test('bootstrap with db identifier', function (group) {
  group.test('bootstrap a database with identifier', function (t) {
    startServer(serverOptions, function (error, server) {
      t.error(error)

      var bootstrapOptions = {
        name: serverOptions.name,
        bootstrapPath: path.join(__dirname, '../fixtures'),
        config: server.settings.app,
        couchUrl: server.methods.getCouchUrl(),
        log: server.log.bind(server)
      }

      bootstrap(bootstrapOptions, function (error) {
        t.error(error)

        var store = server.methods.getStore({
          name: 'test-identifier',
          auth: 'admin'
        })

        t.equal(store.db._db_name, 'http://0.0.0.0:5999/test-db-name', 'the database name was set correctly')

        store.db.get('bootstrapped-document-for-identifier', function (err, doc) {
          t.error(err)
          t.equal(doc._id, 'bootstrapped-document-for-identifier', 'there is a bootstrapped doc')

          server.stop(function () { t.end() })
        })
      })
    })
  })

  group.end()
})
