var test = require('tap').test

var fixAdminParty = require('../../lib/fix-admin-party')

test('fixAdminParty', function (group) {
  group.test('when session request fails', function (t) {
    fixAdminParty({
      couch: {
        session: function (callback) {
          callback('error')
        }
      }
    }, function (error) {
      t.is(error, 'error', 'returns error')
      t.end()
    })
  })

  group.test('when user is no admin', function (t) {
    fixAdminParty({
      couch: {
        session: function (callback) {
          callback(null, {
            userCtx: {
              roles: ['a-role']
            }
          })
        }
      }
    }, function (error) {
      t.error(error, 'no error returned')
      t.end()
    })
  })

  group.test('when user is admin', function (t) {
    fixAdminParty({
      couch: {
        session: function (callback) {
          callback(null, {
            userCtx: {
              roles: ['_admin']
            }
          })
        },
        request: function (options, callback) {
          t.is(options.method, 'put', 'correct method')
          t.is(options.path, '_config/admins/admin', 'correct path')
          t.is(options.body, 'secure', 'correct body')
          callback()
        }
      },
      log: function (scope, msg) {
        t.is(msg, 'admin user created: admin', 'correct log message send')
      },
      config: {
        admin: {
          username: 'admin',
          password: 'secure'
        }
      }
    }, function (error) {
      t.error(error, 'no error returned')
      t.end()
    })
  })

  group.end()
})
